// This work is licensed under the Creative Commons Attribution-ShareAlike 4.0
// International Public License.
// Visit https://creativecommons.org/licenses/by-sa/4.0/

// Persistence of Vision Ray Tracer version 3.8 Include File. (yuqk fork)
// File: cfunctions.inc
// Last updated: 28 October 2023

// Description: This file contains the cf_<name> function hooks for inbuilt and
// c++ wrapped functions which work with complex numbers. Primarily we are
// wrapping the c++11 support for complex numbers. Sometimes with different
// names to better align with existing POV-Ray SDL and the function VM function
// names. Sometimes we opt for our own implementation over the c++ provided
// one. Further a few functions not available as c++ is shipped have been
// provided.

// Currently, the complex math is limited to float accuracy due encoding the
// real and imaginary values in the space of a passed double so as to fit
// within the structure of the existing function vm.

// It is also true where the c++ functions can often accept non-complex, real
// numbers as arguments, the complex functions herein expect a complex number
// where the imaginary part is set to zero as stand ins for reals.

// Tor Olav Kristensen github SDL code also had equiv of cf_id and cf_pos, but
// I cannot see a reason to code them up as they just return the input complex
// number.

//------------------------------------------------------------------------------

#ifndef(Cfunctions_Inc_Temp)
#declare Cfunctions_Inc_Temp = version;
#version 3.8; // For yuqk adding 'unoffical' makes some sense, but warnings then ugly.

#ifdef(View_POV_Include_Stack)
    #debug "including cfunctions.inc\n"
#end

// Inbuilt function name to internal function hooks follow as well as
// documentation regarding each function.

#declare cf_cmplx = function { internal(89) }

// Encodes passed real and imaginary single float values as complex number.
// The encoding is stored in the space of a double float so limit the two input
// values to single float value ranges! The value returned 'double' can be
// passed around as SDL doubles. It's 'just a little' like having a single
// float accuracy complex type - without the type checking...

// Two parameters.

// 1. Single float accuracy real value.

// 2. Single float accuracy imaginary value.


#declare cf_add = function { internal(90) }

// Adds the two passed complex numbers. Returns a complex number as a result.
// Uses the C++ complex values over load of '+'.

// Two parameters.

// 1. Complex number.

// 2. Complex number.


#declare cf_sub = function { internal(91) }

// Subtracts the second passed complex number from the first. Returns a
// complex number as a result. Uses the C++ complex values over load of '-'.

// Two parameters.

// 1. Complex number.

// 2. Complex number.


#declare cf_mul = function { internal(92) }

// Multiplies the two passed complex numbers. Returns a complex number as a
// result. Uses the C++ complex values over load of '*'.

// Two parameters.

// 1. Complex number.

// 2. Complex number.


#declare cf_div = function { internal(93) }

// Divides the first passed complex number by the second. Returns a complex
// number as a result. Uses the C++ complex values over load of '/'.

// Two parameters.

// 1. Complex number.

// 2. Complex number.


#declare cf_real = function { internal(94) }

// Returns the real part of passed complex number. Necessarily returned as a
// double due the vm set up. Equivalent to C++ complex std::real().

// One parameter.

// 1. Complex number.


#declare cf_imag = function { internal(95) }

// Returns the imaginary part of passed complex number. Necessarily returned
// as a double due the vm set up. Equivalent to C++ complex std::image().

// One parameter.

// 1. Complex number.


#declare cf_abs = function { internal(96) }

// Returns the magnitude of a complex number (rho). Necessarily returned as a
// double due the vm set up. Equivalent to C++ complex std::abs().

// One parameter.

// 1. Complex number.


#declare cf_phase = function { internal(97) }

// Returns the phase angle (theta). Necessarily returned as a double due the
// vm set up. Equivalent to C++ complex std::arg(), but herein using cf_phase.

// One parameter.

// 1. Complex number.


#declare cf_conj = function { internal(98) }

// Returns the complex conjugate of the complex number. The returned value is
// an encoded complex in a double value space due the vm set up. Equivalent to
// C++ complex std::conj().

// One parameter.

// 1. Complex number.


#declare cf_norm = function { internal(99) }

// Returns the projection onto the Riemann sphere the complex number. Returns
// a double due the vm set up. Equivalent to C++ complex std::norm().

// One parameter.

// 1. Complex number.


#declare cf_proj = function { internal(100) }

// Returns the squared magnitude of the complex number. The returned value is
// an encoded complex in a double value space due the vm set up. Equivalent to
// C++ complex std::proj(). Note! This a standardization of infinites which can
// be done ahead of a comparison of two complex numbers. It may not have much
// value in debug compiles where cf_cmplx prevents such float values.

// NOTE! It's also true this functionality FAILs for compiles using the g++
// -ffast-math flag !! We may need to add a 'fast-math' configuration option
// the autotools build process - there are a couple other places in the code
// which might alter form on such a configuration.

// One parameter.

// 1. Complex number.


#declare cf_polar = function { internal(101) }

// Constructs a complex number from magnitude and phase angle. Returns an two
// float encoded complex number as a result. Uses the C++ complex function
// polar.

// FWIW. Calculations are being done at single float accuracy and this leads to
// occasional differences in the polarity of any returned 0 over what would be
// seen with calculations at double. Leaving at float as don't think it makes a
// practical difference - but results at float are different than what examples
// at doubles will show.

// Three parameters.

// 1. Magnitude.

// 2. Phase angle.

// 3. If 0, passed phase angle(2) in radians. If 1, phase angle(2) is in
// degrees.


#declare cf_exp = function { internal(102) }

// Compute base-e exponential of z. That is e (Euler's number, 2.7182818)
// raised to the z power. Returns a complex number as a results encoded as two
// floats in double's space due the vm set up. Equivalent to C++ complex
// std::exp().

// One parameter.

// 1. Complex number.


#declare cf_ln = function { internal(103) }

// Computes complex natural (base e) logarithm of a complex value z. It does
// this with a branch cut along the negative real axis. Returns a complex
// number as a results encoded as two floats in double's space due the vm set
// up.  Equivalent to C++ complex std::log(). The cf_ln() name chosen to align
// with SDL natural log function names.

// One parameter.

// 1. Complex number.


#declare cf_log = function { internal(104) }

// Computes complex common (base 10) logarithm of a complex value z. Performed
// with a branch cut along the negative real axis. Returns a complex number as
// a results encoded as two floats in double's space due the vm set up.
// Equivalent to C++ complex std::log10(). The cf_log() name chosen to align
// with SDL natural log function names.

// One parameter.

// 1. Complex number.


#declare cf_pow = function { internal(105) }

// Complex power function. Both arguments must be a complex number, but reals
// exponents can be represented by setting the imaginary portion to zero.
// Returns a complex number as a results encoded as two floats in double's
// space due the vm set up. Equivalent to C++ complex std::pow().

// One parameter.

// 1. Complex number.


#declare cf_sqrt = function { internal(106) }

// Computes the square root of the complex number z. Done with a branch cut
// along the negative real axis. Returns a complex number as a results encoded
// as two floats in double's space due the vm set up. Equivalent to C++
// complex's std::sqrt().

// One parameter.

// 1. Complex number.


#declare cf_sqr = function { internal(107) }

// Computes the square of the complex number. Returns a complex number as a
// results encoded as two floats in double's space due the vm set up.
// Equivalent to C++ complex 'Z*Z'.

// One parameter.

// 1. Complex number.


#declare cf_inv = function { internal(108) }

// Computes the 1/Z inverse of the complex number. Returns a complex number as
// a results encoded as two floats in double's space due the vm set up.
// Equivalent to C++ complex '1/Z'.

// One parameter.

// 1. Complex number.


#declare cf_neg = function { internal(109) }

// Negated complex number. Returns a complex number as a results encoded as
// two floats in double's space due the vm set up. Equivalent to (-real,
// -imag).

// One parameter.

// 1. Complex number.


#declare cf_sin = function { internal(110) }

// Computes sine of a complex number (sin z). Returns a complex number as a
// results encoded as two floats in double's space due the vm set up.
// Equivalent to C++ complex's std::sin(z).

// One parameter.

// 1. Complex number.


#declare cf_cos = function { internal(111) }

// Computes cosine of a complex number (cos z). Returns a complex number as a
// results encoded as two floats in double's space due the vm set up.
// Equivalent to C++ complex's std::cos(z).

// One parameter.

// 1. Complex number.


#declare cf_tan = function { internal(112) }

// Computes tangent of a complex number (tan z). Returns a complex number as a
// results encoded as two floats in double's space due the vm set up.
// Equivalent to C++ complex's std::tan(z).

// One parameter.

// 1. Complex number.


#declare cf_asin = function { internal(113) }

// Computes arc sine of a complex number (asin z). Returns a complex number as
// a results encoded as two floats in double's space due the vm set up.
// Equivalent to C++ complex's std::asin(z).

// One parameter.

// 1. Complex number.


#declare cf_acos = function { internal(114) }

// Computes arc cosine of a complex number (acos z). Returns a complex number
// as a results encoded as two floats in double's space due the vm set up.
// Equivalent to C++ complex's std::acos(z).

// One parameter.

// 1. Complex number.


#declare cf_atan = function { internal(115) }

// Computes arc tangent of a complex number (atan z). Returns a complex number
// as a results encoded as two floats in double's space due the vm set up.
// Equivalent to C++ complex's std::atan(z).

// One parameter.

// 1. Complex number.


#declare cf_sinh = function { internal(116) }

// Computes hyperbolic sine of a complex number (sinh z). Returns a complex
// number as a results encoded as two floats in double's space due the vm set
// up. Equivalent to C++ complex's std::sinh(z).

// One parameter.

// 1. Complex number.


#declare cf_cosh = function { internal(117) }

// Computes hyperbolic cosine of a complex number (cosh z). Returns a complex
// number as a results encoded as two floats in double's space due the vm set
// up. Equivalent to C++ complex's std::cosh(z).

// One parameter.

// 1. Complex number.


#declare cf_tanh = function { internal(118) }

// Computes hyperbolic tangent of a complex number (tanh z). Returns a complex
// number as a results encoded as two floats in double's space due the vm set
// up. Equivalent to C++ complex's std::tanh(z).

// One parameter.

// 1. Complex number.


#declare cf_asinh = function { internal(119) }

// Computes arc hyperbolic sine of a complex number (asinh z). Returns a
// complex number as a results encoded as two floats in double's space due the
// vm set up.  Equivalent to C++ complex's std::asinh(z).

// One parameter.

// 1. Complex number.


#declare cf_acosh = function { internal(120) }

// Computes arc hyperbolic cosine of a complex number (acosh z). Returns a
// complex number as a results encoded as two floats in double's space due the
// vm set up. Equivalent to C++ complex's std::acosh(z).

// One parameter.

// 1. Complex number.


#declare cf_atanh = function { internal(121) }

// Computes arc hyperbolic tangent of a complex number (atanh z). Returns a
// complex number as a results encoded as two floats in double's space due the
// vm set up. Equivalent to C++ complex's std::atanh(z).

// One parameter.

// 1. Complex number.


// STOP_SIGN

//================================= End render time declarations ===================================

//--- Self testing:  yuqk +mv3.8 +i<this_file>
#if(input_file_name="cfunctions.inc")

    #fopen FID "SelfTestCfunctionsInc.txt" write
    #write (FID,"\nExpect value followed by value generated.\nF_ prefixed are pattern/math wrapped equivalents.\n")
    // cf_add
    #write (FID,"\n---\n")
    #write (FID,concat("cf_add 0.123000 ",str(cf_real(cf_add(cf_cmplx(0.123,0),cf_cmplx(0,0.456))),9,6)," r \n"))
    #write (FID,concat("cf_add 0.456000 ",str(cf_imag(cf_add(cf_cmplx(0.123,0),cf_cmplx(0,0.456))),9,6)," i \n"))
    #write (FID,concat("cf_add 0.000000 ",str(cf_real(cf_add(cf_cmplx(+2e-38,+2e-38),cf_cmplx(0,0))),9,6)," r \n"))
    #write (FID,concat("cf_add -0.000000 ",str(cf_imag(cf_add(cf_cmplx(-2e-38,-2e-38),cf_cmplx(0,0))),9,6)," i \n"))
    #write (FID,"\n")
    // cf_sub
    #write (FID,concat("cf_sub 0.123000 ",str(cf_real(cf_sub(cf_cmplx(0.123,0),cf_cmplx(0,0.456))),9,6)," r \n"))
    #write (FID,concat("cf_sub -0.456000 ",str(cf_imag(cf_sub(cf_cmplx(0.123,0),cf_cmplx(0,0.456))),9,6)," i \n"))
    #write (FID,"\n")
    // cf_mul
    #write (FID,concat("cf_mul -1.000000 ",str(cf_real(cf_mul(cf_cmplx(0,1),cf_cmplx(0,1))),9,6)," r \n"))
    #write (FID,concat("cf_mul  0.000000 ",str(cf_imag(cf_mul(cf_cmplx(0,1),cf_cmplx(0,1))),9,6)," i \n"))
    #write (FID,"\n")
    // cf_div
    #write (FID,concat("cf_div  0.000000 ",str(cf_real(cf_div(cf_cmplx(1,0),cf_cmplx(0,1))),9,6)," r \n"))
    #write (FID,concat("cf_div -1.000000 ",str(cf_imag(cf_div(cf_cmplx(1,0),cf_cmplx(0,1))),9,6)," i \n"))
    #write (FID,"\n")
    // cf_abs
    #write (FID,concat("cf_abs  1.414214 ",str(cf_abs(cf_cmplx(1,1)),9,6)," rho \n"))
    #write (FID,"\n")
    // cf_phase
    #write (FID,concat("cf_phase  0.785398 ",str(cf_phase(cf_cmplx(1,1)),9,6)," theta \n"))
    #write (FID,"\n")
    // cf_conj
    #write (FID,concat("cf_conj  3.000000 ",str(cf_real(cf_conj(cf_cmplx(3,4))),9,6)," conjugate of complex r \n"))
    #write (FID,concat("cf_conj  -4.000000 ",str(cf_imag(cf_conj(cf_cmplx(3,4))),9,6)," conjugate of complex i \n"))
    #write (FID,"\n")
    // cf_norm
    #write (FID,concat("cf_norm  25.000000 ",str(cf_norm(cf_cmplx(3,4)),9,6)," squared magnitude \n"))
    #write (FID,"\n")
    // cf_proj   Unsure how useful as the inf standardization doesn't work if using -ffast-math.
    #write (FID,concat("cf_proj  2 ",str(cf_real(cf_proj(cf_cmplx(2,3))),9,6)," proj on Riemann sphere r \n"))
    #write (FID,concat("cf_proj  3 ",str(cf_imag(cf_proj(cf_cmplx(2,3))),9,6)," proj on Riemann sphere i \n"))
    //--- Unable to run the following options with yuqk because POV_DEBUG debug prevents arguments > single float size.
    //--- Using -ffast-math also corrupts ability for cf_proj to handle inf cases for which it is most intended...
    //--- TODO  Should yuqk have a yes/no ffast-math configure option. There is that bounding hack/fix too that
    //--- could configured in/out
    #write (FID,concat("Incomplete_Testing 1.000000 ",str(0.0,9,6)," cf_proj inf cases only work without POV_DEBUG and -fnofast-math\n"))
    //#write (FID,concat("cf_proj  inf ",str(cf_real(cf_proj(cf_cmplx(1/1e-100,-1))),9,6)," proj on Riemann sphere r \n"))
    //#write (FID,concat("cf_proj  -0.000000 ",str(cf_imag(cf_proj(cf_cmplx(1/1e-100,-1))),9,6)," proj on Riemann sphere i \n"))
    //#write (FID,concat("cf_proj  inf ",str(cf_real(cf_proj(cf_cmplx(0,-1/1e-100))),9,6)," proj on Riemann sphere r \n"))
    //#write (FID,concat("cf_proj  -0.000000 ",str(cf_imag(cf_proj(cf_cmplx(0,-1/1e-100))),9,6)," proj on Riemann sphere i \n"))
    #write (FID,"\n")
    // cf_polar   NOTE! FWIW. The polarity of zeros will differ from below due float vs double calcs.
    #write (FID,concat("cf_polar  1.000000 ",str(cf_real(cf_polar(1,0,1)),9,6)," 0 degrees r \n"))
    #write (FID,concat("cf_polar +0.000000 ",str(cf_imag(cf_polar(1,0,1)),9,6)," 0 degrees i \n"))
    #write (FID,concat("cf_polar +0.000000 ",str(cf_real(cf_polar(1,90,1)),9,6)," 90 degrees r \n"))
    #write (FID,concat("cf_polar +1.000000 ",str(cf_imag(cf_polar(1,90,1)),9,6)," 90 degrees i \n"))
    #write (FID,concat("cf_polar -1.000000 ",str(cf_real(cf_polar(1,180,1)),9,6)," 180 degrees r \n"))
    #write (FID,concat("cf_polar +0.000000 ",str(cf_imag(cf_polar(1,180,1)),9,6)," 180 degrees i \n"))
    #write (FID,concat("cf_polar -0.000000 ",str(cf_real(cf_polar(1,270,1)),9,6)," 270 degrees r \n"))
    #write (FID,concat("cf_polar -1.000000 ",str(cf_imag(cf_polar(1,270,1)),9,6)," 270 degrees i \n"))
    #write (FID,concat("cf_polar -0.000000 ",str(cf_real(cf_polar(1,-90,1)),9,6)," -90 degrees r \n"))
    #write (FID,concat("cf_polar -1.000000 ",str(cf_imag(cf_polar(1,-90,1)),9,6)," -90 degrees i \n"))
    #write (FID,concat("cf_polar +0.707107 ",str(cf_real(cf_polar(1,+45,1)),9,6)," +45 degrees r \n"))
    #write (FID,concat("cf_polar +0.707107 ",str(cf_imag(cf_polar(1,+45,1)),9,6)," +45 degrees i \n"))
    #write (FID,"\n")
    // cf_exp
    #write (FID,concat("cf_exp -1.000000 ",str(cf_real(cf_exp(cf_mul(cf_cmplx(0,1),cf_cmplx(pi,0)))),9,6)," r \n"))
    #write (FID,concat("cf_exp +0.000000 ",str(cf_imag(cf_exp(cf_mul(cf_cmplx(0,1),cf_cmplx(pi,0)))),9,6)," i \n"))
    #write (FID,"\n")
    // cf_ln
    #write (FID,concat("cf_ln  0.000000 ",str(cf_real(cf_ln(cf_cmplx(0,1))),9,6)," r \n"))
    #write (FID,concat("cf_ln +1.570796 ",str(cf_imag(cf_ln(cf_cmplx(0,1))),9,6)," i \n"))
    #write (FID,concat("cf_ln  0.000000 ",str(cf_real(cf_mul(cf_cmplx(2,0),cf_ln(cf_cmplx(0,1)))),9,6)," r \n"))
    #write (FID,concat("cf_ln +3.141593 ",str(cf_imag(cf_mul(cf_cmplx(2,0),cf_ln(cf_cmplx(0,1)))),9,6)," i \n"))
    #write (FID,concat("cf_ln  0.000000 ",str(cf_real(cf_ln(cf_cmplx(-1,0))),9,6)," r +0 \n"))
    #write (FID,concat("cf_ln +3.141593 ",str(cf_imag(cf_ln(cf_cmplx(-1,0))),9,6)," i +0 \n"))
    #write (FID,concat("cf_ln  0.000000 ",str(cf_real(cf_ln(cf_cmplx(-1,-0))),9,6)," r -0 (other side of cut) \n"))
    #write (FID,concat("cf_ln -3.141593 ",str(cf_imag(cf_ln(cf_cmplx(-1,-0))),9,6)," i -0 (other side of cut) \n"))
    #write (FID,"\n")
    // cf_log
    #write (FID,concat("cf_log  0.000000 ",str(cf_real(cf_log(cf_cmplx(0,1))),9,6)," r \n"))
    #write (FID,concat("cf_log +0.682188 ",str(cf_imag(cf_log(cf_cmplx(0,1))),9,6)," i \n"))
    #write (FID,concat("cf_log  0.000000 ",str(cf_real(cf_mul(cf_cmplx(2,0),cf_log(cf_cmplx(0,1)))),9,6)," r \n"))
    #write (FID,concat("cf_log +1.364376 ",str(cf_imag(cf_mul(cf_cmplx(2,0),cf_log(cf_cmplx(0,1)))),9,6)," i \n"))
    #write (FID,concat("cf_log  2.000000 ",str(cf_real(cf_log(cf_cmplx(-100,0))),9,6)," r +0 \n"))
    #write (FID,concat("cf_log +1.364376 ",str(cf_imag(cf_log(cf_cmplx(-100,0))),9,6)," i +0 \n"))
    #write (FID,concat("cf_log  2.000000 ",str(cf_real(cf_log(cf_cmplx(-100,-0))),9,6)," r -0 (other side of cut) \n"))
    #write (FID,concat("cf_log -1.364376 ",str(cf_imag(cf_log(cf_cmplx(-100,-0))),9,6)," i -0 (other side of cut) \n"))
    #write (FID,"\n")
    // cf_pow
    #write (FID,concat("cf_pow -3.000000 ",str(cf_real(cf_pow(cf_cmplx(1,2),cf_cmplx(2,0))),9,6)," r \n"))
    #write (FID,concat("cf_pow +4.000000 ",str(cf_imag(cf_pow(cf_cmplx(1,2),cf_cmplx(2,0))),9,6)," i \n"))
    #write (FID,concat("cf_pow  0.000000 ",str(cf_real(cf_pow(cf_cmplx(-1,0),cf_cmplx(0.5,0))),9,6)," r eqv sqrt\n"))
    #write (FID,concat("cf_pow +1.000000 ",str(cf_imag(cf_pow(cf_cmplx(-1,0),cf_cmplx(0.5,0))),9,6)," i eqv sqrt\n"))
    #write (FID,concat("cf_pow  0.000000 ",str(cf_real(cf_pow(cf_cmplx(-1,-0),cf_cmplx(0.5,0))),9,6)," r eqv sqrt oth cut\n"))
    #write (FID,concat("cf_pow -1.000000 ",str(cf_imag(cf_pow(cf_cmplx(-1,-0),cf_cmplx(0.5,0))),9,6)," i eqv sqrt oth cut\n"))
    #write (FID,concat("cf_pow  0.207880 ",str(cf_real(cf_pow(cf_cmplx(0,1),cf_cmplx(0,1))),9,6)," r i^i\n"))
    #write (FID,concat("cf_pow  0.000000 ",str(cf_imag(cf_pow(cf_cmplx(0,1),cf_cmplx(0,1))),9,6)," i i^i\n"))
    #write (FID,"\n")
    // cf_sqrt
    #write (FID,concat("cf_sqrt  0.000000 ",str(cf_real(cf_sqrt(cf_cmplx(-4,0))),9,6)," r \n"))
    #write (FID,concat("cf_sqrt +2.000000 ",str(cf_imag(cf_sqrt(cf_cmplx(-4,0))),9,6)," i \n"))
    #write (FID,concat("cf_sqrt  0.000000 ",str(cf_real(cf_sqrt(cf_cmplx(-4,-0))),9,6)," r othr cut\n"))
    #write (FID,concat("cf_sqrt -2.000000 ",str(cf_imag(cf_sqrt(cf_cmplx(-4,-0))),9,6)," i othr cut\n"))
    #write (FID,"\n")
    // cf_sqr
    #write (FID,concat("cf_sqr  0.000000 ",str(cf_real(cf_sqr(cf_cmplx(1,1))),9,6)," r \n"))
    #write (FID,concat("cf_sqr +2.000000 ",str(cf_imag(cf_sqr(cf_cmplx(1,1))),9,6)," i \n"))
    #write (FID,"\n")
    // cf_inv
    #write (FID,concat("cf_inv +0.500000 ",str(cf_real(cf_inv(cf_cmplx(1,1))),9,6)," r \n"))
    #write (FID,concat("cf_inv -0.500000 ",str(cf_imag(cf_inv(cf_cmplx(1,1))),9,6)," i \n"))
    #write (FID,"\n")
    // cf_neg
    #write (FID,concat("cf_neg -1.000000 ",str(cf_real(cf_neg(cf_cmplx(1,1))),9,6)," r \n"))
    #write (FID,concat("cf_neg -1.000000 ",str(cf_imag(cf_neg(cf_cmplx(1,1))),9,6)," i \n"))
    #write (FID,"\n")
    // cf_sin
    #write (FID,concat("cf_sin  0.841471 ",str(cf_real(cf_sin(cf_cmplx(1,0))),9,6)," r \n"))
    #write (FID,concat("cf_sin  0.000000 ",str(cf_imag(cf_sin(cf_cmplx(1,0))),9,6)," i \n"))
    #write (FID,concat("cf_sin  0.000000 ",str(cf_real(cf_sin(cf_cmplx(0,1))),9,6)," r \n"))
    #write (FID,concat("cf_sin  1.175201 ",str(cf_imag(cf_sin(cf_cmplx(0,1))),9,6)," i \n"))
    #write (FID,"\n")
    // cf_cos
    #write (FID,concat("cf_cos  0.540302 ",str(cf_real(cf_cos(cf_cmplx(1,0))),9,6)," r \n"))
    #write (FID,concat("cf_cos -0.000000 ",str(cf_imag(cf_cos(cf_cmplx(1,0))),9,6)," i \n"))
    #write (FID,concat("cf_cos  1.543081 ",str(cf_real(cf_cos(cf_cmplx(0,1))),9,6)," r \n"))
    #write (FID,concat("cf_cos -0.000000 ",str(cf_imag(cf_cos(cf_cmplx(0,1))),9,6)," i \n"))
    #write (FID,"\n")
    // cf_tan
    #write (FID,concat("cf_tan  1.557408 ",str(cf_real(cf_tan(cf_cmplx(1,0))),9,6)," r \n"))
    #write (FID,concat("cf_tan  0.000000 ",str(cf_imag(cf_tan(cf_cmplx(1,0))),9,6)," i \n"))
    #write (FID,concat("cf_tan  0.000000 ",str(cf_real(cf_tan(cf_cmplx(0,1))),9,6)," r \n"))
    #write (FID,concat("cf_tan  0.761594 ",str(cf_imag(cf_tan(cf_cmplx(0,1))),9,6)," i \n"))
    #write (FID,"\n")
    // cf_asin
    #write (FID,concat("cf_asin -1.570796 ",str(cf_real(cf_asin(cf_cmplx(-2,0))),9,6)," r \n"))
    #write (FID,concat("cf_asin  1.316958 ",str(cf_imag(cf_asin(cf_cmplx(-2,0))),9,6)," i \n"))
    #write (FID,concat("cf_asin -1.570796 ",str(cf_real(cf_asin(cf_cmplx(-2,-0))),9,6)," r othr cut\n"))
    #write (FID,concat("cf_asin -1.316958 ",str(cf_imag(cf_asin(cf_cmplx(-2,-0))),9,6)," i othr cut \n"))
    #write (FID,"\n")
    // cf_acos
    #write (FID,concat("cf_acos  3.141593 ",str(cf_real(cf_acos(cf_cmplx(-2,0))),9,6)," r \n"))
    #write (FID,concat("cf_acos -1.316958 ",str(cf_imag(cf_acos(cf_cmplx(-2,0))),9,6)," i \n"))
    #write (FID,concat("cf_acos  3.141593 ",str(cf_real(cf_acos(cf_cmplx(-2,-0))),9,6)," r othr cut\n"))
    #write (FID,concat("cf_acos  1.316958 ",str(cf_imag(cf_acos(cf_cmplx(-2,-0))),9,6)," i othr cut\n"))
    #write (FID,"\n")
    // cf_atan
    #write (FID,concat("cf_atan  1.570796 ",str(cf_real(cf_atan(cf_cmplx(0,2))),9,6)," r \n"))
    #write (FID,concat("cf_atan  0.549306 ",str(cf_imag(cf_atan(cf_cmplx(0,2))),9,6)," i \n"))
    #write (FID,concat("cf_atan -1.570796 ",str(cf_real(cf_atan(cf_cmplx(-0,2))),9,6)," r othr cut\n"))
    #write (FID,concat("cf_atan  0.549306 ",str(cf_imag(cf_atan(cf_cmplx(-0,2))),9,6)," i othr cut\n"))
    #write (FID,"\n")
    // cf_sinh
    #write (FID,concat("cf_sinh  1.175201 ",str(cf_real(cf_sinh(cf_cmplx(1,0))),9,6)," r \n"))
    #write (FID,concat("cf_sinh  0.000000 ",str(cf_imag(cf_sinh(cf_cmplx(1,0))),9,6)," i \n"))
    #write (FID,concat("cf_sinh  0.000000 ",str(cf_real(cf_sinh(cf_cmplx(0,1))),9,6)," r \n"))
    #write (FID,concat("cf_sinh  0.841471 ",str(cf_imag(cf_sinh(cf_cmplx(0,1))),9,6)," i \n"))
    #write (FID,"\n")
    // cf_cosh
    #write (FID,concat("cf_cosh  1.543081 ",str(cf_real(cf_cosh(cf_cmplx(1,0))),9,6)," r \n"))
    #write (FID,concat("cf_cosh  0.000000 ",str(cf_imag(cf_cosh(cf_cmplx(1,0))),9,6)," i \n"))
    #write (FID,concat("cf_cosh  0.540302 ",str(cf_real(cf_cosh(cf_cmplx(0,1))),9,6)," r \n"))
    #write (FID,concat("cf_cosh  0.000000 ",str(cf_imag(cf_cosh(cf_cmplx(0,1))),9,6)," i \n"))
    #write (FID,"\n")
    // cf_tanh
    #write (FID,concat("cf_tanh  0.761594 ",str(cf_real(cf_tanh(cf_cmplx(1,0))),9,6)," r \n"))
    #write (FID,concat("cf_tanh  0.000000 ",str(cf_imag(cf_tanh(cf_cmplx(1,0))),9,6)," i \n"))
    #write (FID,concat("cf_tanh  0.000000 ",str(cf_real(cf_tanh(cf_cmplx(0,1))),9,6)," r \n"))
    #write (FID,concat("cf_tanh  1.557408 ",str(cf_imag(cf_tanh(cf_cmplx(0,1))),9,6)," i \n"))
    #write (FID,"\n")
    // cf_asinh
    #write (FID,concat("cf_asinh  1.316958 ",str(cf_real(cf_asinh(cf_cmplx(0,-2))),9,6)," r \n"))
    #write (FID,concat("cf_asinh -1.570796 ",str(cf_imag(cf_asinh(cf_cmplx(0,-2))),9,6)," i \n"))
    #write (FID,concat("cf_asinh -1.316958 ",str(cf_real(cf_asinh(cf_cmplx(-0,-2))),9,6)," r othr cut\n"))
    #write (FID,concat("cf_asinh -1.570796 ",str(cf_imag(cf_asinh(cf_cmplx(-0,-2))),9,6)," i othr cut \n"))
    #write (FID,concat("cf_asinh  1.469352 ",str(cf_real(cf_asinh(cf_cmplx(1,2))),9,6)," r \n"))
    #write (FID,concat("cf_asinh  1.063440 ",str(cf_imag(cf_asinh(cf_cmplx(1,2))),9,6)," i \n"))
    #write (FID,"\n")
    // cf_acosh
    #write (FID,concat("cf_acosh  0.000000 ",str(cf_real(cf_acosh(cf_cmplx(0.5,0))),9,6)," r \n"))
    #write (FID,concat("cf_acosh  1.047198 ",str(cf_imag(cf_acosh(cf_cmplx(0.5,0))),9,6)," i \n"))
    #write (FID,concat("cf_acosh  0.000000 ",str(cf_real(cf_acosh(cf_cmplx(0.5,-0))),9,6)," r othr cut\n"))
    #write (FID,concat("cf_acosh -1.047198 ",str(cf_imag(cf_acosh(cf_cmplx(0.5,-0))),9,6)," i othr cut\n"))
    #write (FID,concat("cf_acosh  1.061275 ",str(cf_real(cf_acosh(cf_cmplx(1,1))),9,6)," r \n"))
    #write (FID,concat("cf_acosh  0.904557 ",str(cf_imag(cf_acosh(cf_cmplx(1,1))),9,6)," i \n"))
    #write (FID,"\n")
    // cf_atanh
    #write (FID,concat("cf_atanh  0.549306 ",str(cf_real(cf_atanh(cf_cmplx(2,0))),9,6)," r \n"))
    #write (FID,concat("cf_atanh  1.570796 ",str(cf_imag(cf_atanh(cf_cmplx(2,0))),9,6)," i \n"))
    #write (FID,concat("cf_atanh  0.549306 ",str(cf_real(cf_atanh(cf_cmplx(2,-0))),9,6)," r othr cut\n"))
    #write (FID,concat("cf_atanh -1.570796 ",str(cf_imag(cf_atanh(cf_cmplx(2,-0))),9,6)," i othr cut\n"))
    #write (FID,concat("cf_atanh  0.173287 ",str(cf_real(cf_atanh(cf_cmplx(1,2))),9,6)," r \n"))
    #write (FID,concat("cf_atanh  1.178097 ",str(cf_imag(cf_atanh(cf_cmplx(1,2))),9,6)," i \n"))


    #write (FID,"\n")
    #fclose FID

    #error "\n\n=== Self testing. Stop before render. ===\n"

#end // End self testing

#version Cfunctions_Inc_Temp;
#end//cfunctions.inc

// SPDX-License-Identifier: CC-BY-SA-4.0
